/***
*
*	Copyright (c) 1996-2001, Valve LLC. All rights reserved.
*	
*	This product contains software technology licensed from Id 
*	Software, Inc. ("Id Technology").  Id Technology (c) 1996 Id Software, Inc. 
*	All Rights Reserved.
*
*   Use, distribution, and modification of this source code and/or resulting
*   object code is restricted to non-commercial enhancements to products from
*   Valve LLC.  All other use, distribution, or modification is prohibited
*   without written permission from Valve LLC.
*
****/
#if !defined( OEM_BUILD ) && !defined( HLDEMO_BUILD )


#include "extdll.h"
#include "util.h"
#include "cbase.h"
#include "monsters.h"
#include "weapons.h"
#include "nodes.h"
#include "player.h"
#include "gamerules.h"
#include "decals.h"

#ifndef CLIENT_DLL
#define BOLT_AIR_VELOCITY       420//2000,1000
#define BOLT_WATER_VELOCITY     256

// UNDONE: Save/restore this?  Don't forget to set classname and LINK_ENTITY_TO_CLASS()
// 
// OVERLOADS SOME ENTVARS:
//
// speed - the ideal magnitude of my velocity
class CCrossbowBolt : public CBaseEntity
{
	void Spawn( void );
	void Precache( void );
	int  Classify ( void );
	void EXPORT BubbleThink( void );
	void EXPORT BoltTouch( CBaseEntity *pOther );
	void EXPORT ExplodeThink( void );

	int m_iTrail;

public:
	static CCrossbowBolt *BoltCreate( void );
};
LINK_ENTITY_TO_CLASS( crossbow_bolt, CCrossbowBolt );

CCrossbowBolt *CCrossbowBolt::BoltCreate( void )
{
	// Create a new entity with CCrossbowBolt private data
	CCrossbowBolt *pBolt = GetClassPtr( (CCrossbowBolt *)NULL );
	pBolt->pev->classname = MAKE_STRING("bolt");
	pBolt->Spawn();

	return pBolt;
}

void CCrossbowBolt::Spawn( )
{
	Precache( );
	pev->movetype = MOVETYPE_FLY;
	pev->solid = SOLID_BBOX;

        //pev->effects |= EF_LIGHT;
        //EMIT_SOUND( ENT(pev), CHAN_VOICE, "weapons/rocket1.wav", 1, 0.5 );


	pev->gravity = 0.5;

        SET_MODEL(ENT(pev), "models/rpgrocket.mdl");

	UTIL_SetOrigin( pev, pev->origin );
	UTIL_SetSize(pev, Vector(0, 0, 0), Vector(0, 0, 0));

	MESSAGE_BEGIN( MSG_BROADCAST, SVC_TEMPENTITY );
		WRITE_BYTE( TE_BEAMFOLLOW );
		WRITE_SHORT(entindex());	// entity
		WRITE_SHORT(m_iTrail );	// model
                WRITE_BYTE( 24 ); // life
                WRITE_BYTE( 4 );  // width
                WRITE_BYTE( 255 );   // r, g, b
                WRITE_BYTE( 255 );   // r, g, b
		WRITE_BYTE( 255 );   // r, g, b
                WRITE_BYTE( 128 );      // brightness
	MESSAGE_END();  // move PHS/PVS data sending into here (SEND_ALL, SEND_PVS, SEND_PHS)

	SetTouch( &CCrossbowBolt::BoltTouch );
	SetThink( &CCrossbowBolt::BubbleThink );
	pev->nextthink = gpGlobals->time + 0.2;

}


void CCrossbowBolt::Precache( )
{
        PRECACHE_MODEL ("models/rpgrocket.mdl");
        //PRECACHE_SOUND("weapons/xbow_hitbod1.wav");
        //PRECACHE_SOUND("weapons/xbow_hitbod2.wav");
        //PRECACHE_SOUND("weapons/xbow_fly1.wav");
        //PRECACHE_SOUND("weapons/xbow_hit1.wav");
        //PRECACHE_SOUND("fvox/beep.wav");
        //m_iTrail = PRECACHE_MODEL("sprites/streak.spr");
        m_iTrail = PRECACHE_MODEL("sprites/smoke.spr");
       // PRECACHE_SOUND("weapons/rocket1.wav");
}


int	CCrossbowBolt :: Classify ( void )
{
	return	CLASS_NONE;
}

void CCrossbowBolt::BoltTouch( CBaseEntity *pOther )
{
        //STOP_SOUND( ENT(pev), CHAN_VOICE, "weapons/rocket1.wav" );
	SetTouch( NULL );
	SetThink( NULL );


        //the sky gobbles up missiles.
        if ( UTIL_PointContents(pev->origin) == CONTENT_SKY )
	{
                UTIL_Remove( this );
		return;
	}


        entvars_t       *pevOwner;
        pevOwner = VARS( pev->owner );




        //RadiusDamage( vecSrc, pevInflictor, pevAttacker, float flDamage, float flRadius, int iClassIgnore, int bitsDamageType );
        ::RadiusDamage( pev->origin, pev, pevOwner, 32, 48, CLASS_NONE, DMG_BLAST | DMG_NEVERGIB );

	if (pOther->pev->takedamage)
	{
		TraceResult tr = UTIL_GetGlobalTrace( );
                //entvars_t       *pevOwner;

                //pevOwner = VARS( pev->owner );

		// UNDONE: this needs to call TraceAttack instead
		ClearMultiDamage( );

                pOther->TraceAttack(pevOwner, gSkillData.plrDmgCrossbowClient, pev->velocity.Normalize(), &tr, DMG_NEVERGIB ); 
  
		ApplyMultiDamage( pev, pevOwner );

		pev->velocity = Vector( 0, 0, 0 );

           //     if ( !g_pGameRules->IsMultiplayer() )
		{
             //           Killed( pev, GIB_NEVER );
		}
	}
	else
	{


//                SetThink( &CCrossbowBolt::SUB_Remove );
  //              pev->nextthink = gpGlobals->time;// this will get changed below if the bolt is allowed to stick in what it hit.


                TraceResult tr;
                tr.vecEndPos = pev->origin;
                tr.pHit = pOther->edict();

                //Arent we doing this client side?
                //UTIL_GunshotDecalTrace( &tr, DECAL_SCORCH1 + RANDOM_LONG( 0, 2 ));
                UTIL_GunshotDecalTrace( &tr, DECAL_SMALLSCORCH1 + RANDOM_LONG( 0, 2 ));
	}




        int iContents = UTIL_PointContents ( pev->origin );
	MESSAGE_BEGIN( MSG_PVS, SVC_TEMPENTITY, pev->origin );
		WRITE_BYTE( TE_EXPLOSION);		
		WRITE_COORD( pev->origin.x );
		WRITE_COORD( pev->origin.y );
		WRITE_COORD( pev->origin.z );
		if (iContents != CONTENTS_WATER)
		{
			WRITE_SHORT( g_sModelIndexFireball );
		}
		else
		{
			WRITE_SHORT( g_sModelIndexWExplosion );
		}
                WRITE_BYTE( RANDOM_LONG(3,6) ); // scale * 10
                WRITE_BYTE( 16  ); // framerate
                WRITE_BYTE( TE_EXPLFLAG_NOPARTICLES );
	MESSAGE_END();


        pev->effects = 0;
                UTIL_Remove( this );
}

void CCrossbowBolt::BubbleThink( void )
{
        pev->nextthink = gpGlobals->time + 0.2;
          
        entvars_t       *pevOwner;
        pevOwner = VARS( pev->owner );
        UTIL_MakeVectors( pevOwner->v_angle + pevOwner->punchangle );

        //UTIL_MakeVectors( pev->v_angle + pev->punchangle );

        //drunk missile pathmaking
        if (pev->waterlevel == 0)
                pev->velocity = ((gpGlobals->v_forward) + (gpGlobals->v_up * RANDOM_FLOAT(-0.2,0.2)) + (gpGlobals->v_right * RANDOM_FLOAT(-0.2,0.2))) * BOLT_AIR_VELOCITY;
        else //under the sea.
        {
                pev->velocity = (gpGlobals->v_forward + (gpGlobals->v_up * RANDOM_FLOAT(-0.1,0.1))) * BOLT_WATER_VELOCITY;
                UTIL_BubbleTrail( pev->origin - pev->velocity * 0.1, pev->origin, 1 );
        }   
}

void CCrossbowBolt::ExplodeThink( void )
{
	int iContents = UTIL_PointContents ( pev->origin );
	
        //pev->dmg = 40;
/*
	MESSAGE_BEGIN( MSG_PVS, SVC_TEMPENTITY, pev->origin );
		WRITE_BYTE( TE_EXPLOSION);		
		WRITE_COORD( pev->origin.x );
		WRITE_COORD( pev->origin.y );
		WRITE_COORD( pev->origin.z );
		if (iContents != CONTENTS_WATER)
		{
			WRITE_SHORT( g_sModelIndexFireball );
		}
		else
		{
			WRITE_SHORT( g_sModelIndexWExplosion );
		}
                WRITE_BYTE( RANDOM_LONG(10,20) ); // scale * 10
                WRITE_BYTE( 16  ); // framerate
		WRITE_BYTE( TE_EXPLFLAG_NONE );
	MESSAGE_END();
  */
	entvars_t *pevOwner;

	if ( pev->owner )
		pevOwner = VARS( pev->owner );
	else
		pevOwner = NULL;

	pev->owner = NULL; // can't traceline attack owner if this is set

        ::RadiusDamage( pev->origin, pev, pevOwner, pev->dmg, 96, CLASS_NONE, DMG_BLAST | DMG_NEVERGIB );

	UTIL_Remove(this);
}
#endif

enum crossbow_e {
	CROSSBOW_IDLE1 = 0,	// full
	CROSSBOW_IDLE2,		// empty
	CROSSBOW_FIDGET1,	// full
	CROSSBOW_FIDGET2,	// empty
	CROSSBOW_FIRE1,		// full
	CROSSBOW_FIRE2,		// reload
	CROSSBOW_FIRE3,		// empty
	CROSSBOW_RELOAD,	// from empty
	CROSSBOW_DRAW1,		// full
	CROSSBOW_DRAW2,		// empty
	CROSSBOW_HOLSTER1,	// full
	CROSSBOW_HOLSTER2,	// empty
};

LINK_ENTITY_TO_CLASS( weapon_crossbow, CCrossbow );

void CCrossbow::Spawn( )
{
	Precache( );
	m_iId = WEAPON_CROSSBOW;
	SET_MODEL(ENT(pev), "models/w_crossbow.mdl");

	m_iDefaultAmmo = CROSSBOW_DEFAULT_GIVE;


	FallInit();// get ready to fall down.
}

int CCrossbow::AddToPlayer( CBasePlayer *pPlayer )
{
	if ( CBasePlayerWeapon::AddToPlayer( pPlayer ) )
	{
		MESSAGE_BEGIN( MSG_ONE, gmsgWeapPickup, NULL, pPlayer->pev );
			WRITE_BYTE( m_iId );
		MESSAGE_END();
		return TRUE;
	}
	return FALSE;
}

void CCrossbow::Precache( void )
{
	PRECACHE_MODEL("models/w_crossbow.mdl");
	PRECACHE_MODEL("models/v_crossbow.mdl");
	PRECACHE_MODEL("models/p_crossbow.mdl");

	PRECACHE_SOUND("weapons/xbow_fire1.wav");
	PRECACHE_SOUND("weapons/xbow_reload1.wav");

	UTIL_PrecacheOther( "crossbow_bolt" );

	m_usCrossbow = PRECACHE_EVENT( 1, "events/crossbow1.sc" );
	m_usCrossbow2 = PRECACHE_EVENT( 1, "events/crossbow2.sc" );
}


int CCrossbow::GetItemInfo(ItemInfo *p)
{
	p->pszName = STRING(pev->classname);
	p->pszAmmo1 = "uranium";
	p->iMaxAmmo1 = URANIUM_MAX_CARRY;
	p->pszAmmo2 = NULL;
	p->iMaxAmmo2 = -1;
	p->iMaxClip = WEAPON_NOCLIP;
	p->iSlot = 2;
	p->iPosition = 2;
	p->iId = WEAPON_CROSSBOW;
	p->iFlags = 0;
	p->iWeight = CROSSBOW_WEIGHT;
	return 1;
}


BOOL CCrossbow::Deploy( )
{
//        if (m_iClip)
                return DefaultDeploy( "models/v_crossbow.mdl", "models/p_crossbow.mdl", CROSSBOW_DRAW1, "bow" );
//        return DefaultDeploy( "models/v_crossbow.mdl", "models/p_crossbow.mdl", CROSSBOW_DRAW2, "bow" );
}

void CCrossbow::Holster( int skiplocal /* = 0 */ )
{
	m_fInReload = FALSE;// cancel any reload in progress.


	m_pPlayer->m_flNextAttack = UTIL_WeaponTimeBase() + 0.5;
	if (m_iClip)
		SendWeaponAnim( CROSSBOW_HOLSTER1 );
	else
		SendWeaponAnim( CROSSBOW_HOLSTER2 );
}

void CCrossbow::PrimaryAttack( void )
{
//        m_pPlayer->pev->armorvalue += 4;
//        FireBolt();

        m_pPlayer->pev->health--;
         m_flNextPrimaryAttack = UTIL_WeaponTimeBase() + 1;

}



//bc
//void CCrossbow::FireBolt()
//CGrenade *CGrenade::ShootContact( entvars_t *pevOwner, Vector vecStart, Vector vecVelocity )
CCrossbow *CCrossbow :: FireBolt( entvars_t *pevOwner, Vector vecStart, Vector vecVelocity )
{
#ifndef CLIENT_DLL
	CCrossbowBolt *pBolt = CCrossbowBolt::BoltCreate();
        pBolt->pev->origin = vecStart;
        pBolt->pev->owner = ENT(pevOwner);
        pBolt->pev->velocity = vecVelocity;
        pBolt->pev->angles = UTIL_VecToAngles( pBolt->pev->velocity );
        //pBolt->pev->speed = BOLT_AIR_VELOCITY;
        //pBolt->pev->avelocity.z = 10;
#endif
}


void CCrossbow::SecondaryAttack()
{
        if (! (m_pPlayer->m_afButtonPressed & IN_ATTACK2))
        return;

        ALERT(at_console,"FHQWHGADS I SEE YOU\n");

	pev->nextthink = UTIL_WeaponTimeBase() + 0.1;
	m_flNextSecondaryAttack = UTIL_WeaponTimeBase() + 1.0;
}


void CCrossbow::Reload( void )
{
}


void CCrossbow::WeaponIdle( void )
{
}



class CCrossbowAmmo : public CBasePlayerAmmo
{
	void Spawn( void )
	{ 
		Precache( );
		SET_MODEL(ENT(pev), "models/w_crossbow_clip.mdl");
		CBasePlayerAmmo::Spawn( );
	}
	void Precache( void )
	{
		PRECACHE_MODEL ("models/w_crossbow_clip.mdl");
		PRECACHE_SOUND("items/9mmclip1.wav");
	}
	BOOL AddAmmo( CBaseEntity *pOther ) 
	{ 
		if (pOther->GiveAmmo( AMMO_CROSSBOWCLIP_GIVE, "bolts", BOLT_MAX_CARRY ) != -1)
		{
			EMIT_SOUND(ENT(pev), CHAN_ITEM, "items/9mmclip1.wav", 1, ATTN_NORM);
			return TRUE;
		}
		return FALSE;
	}
};
LINK_ENTITY_TO_CLASS( ammo_crossbow, CCrossbowAmmo );



#endif
